@@ -1,6 +1,7 @@
 import os, re, sys, subprocess
 import tarfile
 from distutils import log, sysconfig, version
+from contextlib import closing
 
 try:
     from urlparse import urlsplit, urljoin, unquote
@@ -39,12 +40,16 @@ def download_and_extract_zlatkovic_binaries(destdir):
                     assert fn.endswith('.win32.zip')
                     libs[libname] = fn
     else:
+        if sys.maxsize > 2147483647:
+            arch = "win64"
+        else:
+            arch = "win32"
         url = "https://github.com/mhils/libxml2-win-binaries/releases/download/lxml/"
         libs = dict(
-            libxml2  = "libxml2-latest.win32.zip",
-            libxslt  = "libxslt-latest.win32.zip",
-            zlib     = "zlib-latest.win32.zip",
-            iconv    = "iconv-latest.win32.zip",
+            libxml2  = "libxml2-latest.{}.zip".format(arch),
+            libxslt  = "libxslt-latest.{}.zip".format(arch),
+            zlib     = "zlib-latest.{}.zip".format(arch),
+            iconv    = "iconv-latest.{}.zip".format(arch),
         )
 
     if not os.path.exists(destdir): os.makedirs(destdir)
@@ -90,6 +95,7 @@ def unpack_zipfile(zipfn, destdir):
     assert os.path.exists(extracted_dir), 'missing: %s' % extracted_dir
     return extracted_dir
 
+
 def get_prebuilt_libxml2xslt(download_dir, static_include_dirs, static_library_dirs):
     assert sys.platform.startswith('win')
     libs = download_and_extract_zlatkovic_binaries(download_dir)
@@ -106,26 +112,45 @@ def get_prebuilt_libxml2xslt(download_dir, static_include_dirs, static_library_d
 
 LIBXML2_LOCATION = 'ftp://xmlsoft.org/libxml2/'
 LIBICONV_LOCATION = 'ftp://ftp.gnu.org/pub/gnu/libiconv/'
+ZLIB_LOCATION = 'http://zlib.net/'
 match_libfile_version = re.compile('^[^-]*-([.0-9-]+)[.].*').match
 
-def ftp_listdir(url):
-    assert url.lower().startswith('ftp://')
+
+def _find_content_encoding(response, default='iso8859-1'):
     from email.message import Message
-    res = urlopen(url)
-    content_type = res.headers.get('Content-Type')
+    content_type = response.headers.get('Content-Type')
     if content_type:
         msg = Message()
         msg.add_header('Content-Type', content_type)
-        charset = msg.get_content_charset('utf-8')
+        charset = msg.get_content_charset(default)
     else:
-        charset = 'utf-8'
+        charset = default
+    return charset
+
+
+def ftp_listdir(url):
+    assert url.lower().startswith('ftp://')
+    with closing(urlopen(url)) as res:
+        charset = _find_content_encoding(res)
+        content_type = res.headers.get('Content-Type')
+        data = res.read()
+
+    data = data.decode(charset)
     if content_type and content_type.startswith('text/html'):
-        files = parse_html_ftplist(res.read().decode(charset))
+        files = parse_html_ftplist(data)
     else:
-        files = parse_text_ftplist(res.read().decode(charset))
-    res.close()
+        files = parse_text_ftplist(data)
     return files
 
+
+def http_listfiles(url, re_pattern):
+    with closing(urlopen(url)) as res:
+        charset = _find_content_encoding(res)
+        data = res.read()
+    files = re.findall(re_pattern, data.decode(charset))
+    return files
+
+
 def parse_text_ftplist(s):
     for line in s.splitlines():
         if not line.startswith('d'):
@@ -134,6 +159,7 @@ def parse_text_ftplist(s):
             # may be variations, so we discard only the first 8 entries.
             yield line.split(None, 8)[-1]
 
+
 def parse_html_ftplist(s):
     re_href = re.compile(r'<a\s+(?:[^>]*?\s+)?href=["\'](.*?)[;\?"\']', re.I|re.M)
     links = set(re_href.findall(s))
@@ -141,12 +167,14 @@ def parse_html_ftplist(s):
         if not link.endswith('/'):
             yield unquote(link)
 
+
 def tryint(s):
     try:
         return int(s)
     except ValueError:
         return s
 
+
 def download_libxml2(dest_dir, version=None):
     """Downloads libxml2, returning the filename where the library was downloaded"""
     version_re = re.compile(r'LATEST_LIBXML2_IS_([0-9.]+[0-9])')
@@ -154,6 +182,7 @@ def download_libxml2(dest_dir, version=None):
     return download_library(dest_dir, LIBXML2_LOCATION, 'libxml2',
                             version_re, filename, version=version)
 
+
 def download_libxslt(dest_dir, version=None):
     """Downloads libxslt, returning the filename where the library was downloaded"""
     version_re = re.compile(r'LATEST_LIBXSLT_IS_([0-9.]+[0-9])')
@@ -161,6 +190,7 @@ def download_libxslt(dest_dir, version=None):
     return download_library(dest_dir, LIBXML2_LOCATION, 'libxslt',
                             version_re, filename, version=version)
 
+
 def download_libiconv(dest_dir, version=None):
     """Downloads libiconv, returning the filename where the library was downloaded"""
     version_re = re.compile(r'^libiconv-([0-9.]+[0-9]).tar.gz$')
@@ -168,11 +198,22 @@ def download_libiconv(dest_dir, version=None):
     return download_library(dest_dir, LIBICONV_LOCATION, 'libiconv',
                             version_re, filename, version=version)
 
-def download_library(dest_dir, location, name, version_re, filename,
-                     version=None):
+
+def download_zlib(dest_dir, version):
+    """Downloads zlib, returning the filename where the library was downloaded"""
+    version_re = re.compile(r'zlib-([0-9.]+[0-9]).tar.gz')
+    filename = 'zlib-%s.tar.gz'
+    return download_library(dest_dir, ZLIB_LOCATION, 'zlib',
+                            version_re, filename, version=version)
+
+
+def download_library(dest_dir, location, name, version_re, filename, version=None):
     if version is None:
         try:
-            fns = ftp_listdir(location)
+            if location.startswith('ftp://'):
+                fns = ftp_listdir(location)
+            else:
+                fns = http_listfiles(location, filename.replace('%s', '(?:[0-9.]+[0-9])'))
             versions = []
             for fn in fns:
                 match = version_re.search(fn)
@@ -253,10 +294,12 @@ def call_subprocess(cmd, **kw):
     if returncode:
         raise Exception('Command "%s" returned code %s' % (cmd_desc, returncode))
 
+
 def safe_mkdir(dir):
     if not os.path.exists(dir):
         os.makedirs(dir)
 
+
 def cmmi(configure_cmd, build_dir, multicore=None, **call_setup):
     print('Starting build in %s' % build_dir)
     call_subprocess(configure_cmd, cwd=build_dir, **call_setup)
@@ -273,6 +316,7 @@ def cmmi(configure_cmd, build_dir, multicore=None, **call_setup):
         ['make'] + make_jobs + ['install'],
         cwd=build_dir, **call_setup)
 
+
 def configure_darwin_env(env_setup):
     import platform
     # check target architectures on MacOS-X (ppc, i386, x86_64)
@@ -293,27 +337,30 @@ def configure_darwin_env(env_setup):
             arch_string = "-arch ppc "
         if minor_version < 6:
             env_default = {
-                'CFLAGS' : arch_string + "-arch i386 -isysroot /Developer/SDKs/MacOSX10.4u.sdk -O2",
-                'LDFLAGS' : arch_string + "-arch i386 -isysroot /Developer/SDKs/MacOSX10.4u.sdk",
-                'MACOSX_DEPLOYMENT_TARGET' : "10.3"
+                'CFLAGS': arch_string + "-arch i386 -isysroot /Developer/SDKs/MacOSX10.4u.sdk -O2",
+                'LDFLAGS': arch_string + "-arch i386 -isysroot /Developer/SDKs/MacOSX10.4u.sdk",
+                'MACOSX_DEPLOYMENT_TARGET': "10.3"
             }
         else:
             env_default = {
-                'CFLAGS' : arch_string + "-arch i386 -arch x86_64 -O2",
-                'LDFLAGS' : arch_string + "-arch i386 -arch x86_64",
-                'MACOSX_DEPLOYMENT_TARGET' : "10.6"
+                'CFLAGS': arch_string + "-arch i386 -arch x86_64 -O2",
+                'LDFLAGS': arch_string + "-arch i386 -arch x86_64",
+                'MACOSX_DEPLOYMENT_TARGET': "10.6"
             }
         env = os.environ.copy()
         env_default.update(env)
         env_setup['env'] = env_default
 
+
 def build_libxml2xslt(download_dir, build_dir,
                       static_include_dirs, static_library_dirs,
                       static_cflags, static_binaries,
                       libxml2_version=None, libxslt_version=None, libiconv_version=None,
+                      zlib_version=None,
                       multicore=None):
     safe_mkdir(download_dir)
     safe_mkdir(build_dir)
+    zlib_dir = unpack_tarball(download_zlib(download_dir, zlib_version), build_dir)
     libiconv_dir = unpack_tarball(download_libiconv(download_dir, libiconv_version), build_dir)
     libxml2_dir  = unpack_tarball(download_libxml2(download_dir, libxml2_version), build_dir)
     libxslt_dir  = unpack_tarball(download_libxslt(download_dir, libxslt_version), build_dir)
@@ -330,13 +377,22 @@ def build_libxml2xslt(download_dir, build_dir,
                      '--prefix=%s' % prefix,
                      ]
 
+    # build zlib
+    zlib_configure_cmd = [
+        './configure',
+        '--prefix=%s' % prefix,
+    ]
+    cmmi(zlib_configure_cmd, zlib_dir, multicore, **call_setup)
+
     # build libiconv
     cmmi(configure_cmd, libiconv_dir, multicore, **call_setup)
 
     # build libxml2
     libxml2_configure_cmd = configure_cmd + [
         '--without-python',
-        '--with-iconv=%s' % prefix]
+        '--with-iconv=%s' % prefix,
+        '--with-zlib=%s' % prefix,
+    ]
     try:
         if libxml2_version and tuple(map(tryint, libxml2_version.split('.'))) >= (2,7,3):
             libxml2_configure_cmd.append('--enable-rebuild-docs=no')
@@ -369,7 +425,7 @@ def build_libxml2xslt(download_dir, build_dir,
 
     listdir = os.listdir(lib_dir)
     static_binaries += [os.path.join(lib_dir, filename)
-        for lib in ['libxml2', 'libexslt', 'libxslt', 'iconv']
+        for lib in ['libxml2', 'libexslt', 'libxslt', 'iconv', 'libz']
         for filename in listdir
         if lib in filename and filename.endswith('.a')]
 
